import os
from urllib.parse import urlsplit


def parse_pgurl(url):
    '''
    Given a Postgres url, return a dict with keys for user, password,
    host, port, and database.
    '''
    parsed = urlsplit(url)

    return {
        'user': parsed.username,
        'password': parsed.password,
        'database': parsed.path.lstrip('/'),
        'host': parsed.hostname,
        'port': parsed.port or 5432,
    }


def create_database(db_url=None, super_db_url=None):  # pragma: no cover
    '''
    Creates the postgresql database.
    '''
    import psycopg2  # isort:skip

    super_dbdata = parse_pgurl(
        super_db_url or os.getenv(
            'SUPER_DATABASE_URL',
            'psycopg2+postgresql://postgres@localhost:5432/postgres'
        )
    )
    db_url = db_url or os.getenv('DATABASE_URL')
    assert db_url is not None

    dbdata = parse_pgurl(db_url)
    conn = psycopg2.connect(**super_dbdata)
    conn.set_isolation_level(0)
    cur = conn.cursor()

    cur.execute('CREATE DATABASE {}'.format(dbdata['database']))
    cur.close()
    conn.close()


def drop_database(db_url=None, super_db_url=None):  # pragma: no cover
    '''
    Deletes the existing postgresql database.
    '''
    import psycopg2  # isort:skip

    super_dbdata = parse_pgurl(
        super_db_url or os.getenv(
            'SUPER_DATABASE_URL',
            'psycopg2+postgresql://postgres@localhost:5432/postgres'
        )
    )
    db_url = db_url or os.getenv('DATABASE_URL')
    assert db_url is not None

    dbdata = parse_pgurl(db_url)
    conn = psycopg2.connect(**super_dbdata)
    conn.set_isolation_level(0)
    cur = conn.cursor()

    cur.execute('DROP DATABASE IF EXISTS {}'.format(dbdata['database']))
    cur.close()
    conn.close()



import psycopg2

from chilero.pg.test import TEST_DB_SUFFIX
from chilero.pg.utils import drop_database, create_database, parse_pgurl


def pytest_configure():
    db_url = 'postgres://postgres@localhost:5432/chilero_pg_{}'.format(
        TEST_DB_SUFFIX
    )
    # Drop old test database, if exists
    drop_database(db_url)

    # Create a new, empty, test database
    create_database(db_url)

    # Fill the test database with initial structure
    dbdata = parse_pgurl(db_url)
    conn = psycopg2.connect(**dbdata)
    conn.set_isolation_level(0)
    cur = conn.cursor()
    with open('tests/data/db.sql') as f:
        cur.execute(f.read())
    cur.close()
    conn.close()



import asyncio
import json
import random
import string
import sys

from aiohttp import request
from chilero.web.test import WebTestCase

from .application import Application

TEST_DB_SUFFIX = 'test_{}{}{}'.format(
        sys.version_info.major,
        sys.version_info.minor,
        sys.version_info.micro,
    )


class TestCase(WebTestCase):
    application = Application
    settings = {}

    @asyncio.coroutine
    def initialize_application(self):
        return self.application(
            routes=self.routes,
            settings=self.settings,
            loop=self.loop
        )

    def _random_string(self, length=12):
        return ''.join(
            random.choice(string.ascii_lowercase) for i in range(length)
        )

    @asyncio.coroutine
    def _get(self, path, **kwargs):
        resp = yield from request('GET', path, loop=self.loop, **kwargs)
        return resp

    @asyncio.coroutine
    def _get_json(self, path, **kwargs):
        resp = yield from self._get(path, **kwargs)
        jresp = yield from resp.json()
        resp.close()
        return jresp

    @asyncio.coroutine
    def _index(self, path):
        resp = yield from self._get_json(self.full_url(path))
        return resp

    @asyncio.coroutine
    def _create(self, path, data):
        resp = yield from request(
            'POST', self.full_url(path), loop=self.loop,
            data=json.dumps(data)
        )
        return resp

    @asyncio.coroutine
    def _create_and_get(self, path, data, defaults=None):
        defaults = defaults or {}
        defaults.update(data)
        resp = yield from self._create(path, defaults)
        if resp.status != 201:
            return resp, None

        url = resp.headers['Location']

        jresp = yield from self._get_json(url)

        return resp, jresp['body']

    @asyncio.coroutine
    def _patch(self, url, **kwargs):
        resp = yield from request(
            'PATCH', url, loop=self.loop,
            data=json.dumps(kwargs)
        )
        return resp

    @asyncio.coroutine
    def _search(self, path, terms):
        resp = self._get_json(
            self.full_url(
                '{endpoint}?search={keywords}'.format(
                    endpoint=path,
                    keywords='+'.join(terms.split())
                )
            )
        )
        return resp



import asyncio

from chilero.web import Application as BaseApplication

from .connection import get_pool


class Application(BaseApplication):

    settings = {}

    def __init__(self, *args, **kwargs):
        if 'settings' in kwargs:
            self.settings = kwargs.pop('settings')

        super(Application, self).__init__(*args, **kwargs)

    @asyncio.coroutine
    def get_pool(self):
        if not hasattr(self, 'db_pool'):
            self.db_pool = yield from get_pool(
                db_url=self.settings.get('db_url'),
                minsize=self.settings.get('db_pool_min') or 0,
                maxsize=self.settings.get('db_pool_max') or 10
            )

        return self.db_pool

    @asyncio.coroutine
    def close_pool(self):  # pragma: no cover
        yield from self.db_pool.finish()

    @asyncio.coroutine
    def finish(self):  # pragma: no cover
        yield from self.close_pool()

        yield from super(Application, self).finish()
